#include <sys/types.h>
#include <stdio.h>
#include <stdlib.h>
#include <wav.h>

void
usage(char *argv0)
{

	fprintf(stderr, "usage: %s [input file] [output file]\n",
	    argv0);
}

int
main(int argc, char *argv[])
{
#ifdef WITHOUTSIMD
	int i, tmp;
	int16_t *p;
#endif /* WITHOUTSIMD */
	FILE *fi, *fo;
	uint64_t t0, t1;
	wavStorage *st;

	if(argc != 3) {
		usage(argv[0]);
		exit(1);
	}
	if((fi = fopen(argv[1], "r")) == NULL) {
		fprintf(stderr,
		    "%s: No such file or directory\n", argv[1]);
		exit(1);
	}
	if((fo = fopen(argv[2], "w")) == NULL) {
		fprintf(stderr,
		    "%s: No such file or directory\n", argv[2]);
		exit(1);
	}
	if(((st = readWavFile(fi)) == NULL)
	    || (st->info.formatid != 1) || (st->info.samplebit != 16)
	    || (st->info.nchannel != 2)) {
		fprintf(stderr, "%s: Invalid file format\n",
		    argv[1]);
		exit(1);
	}
	rdtsc(t0);
#ifdef WITHOUTSIMD
	for(i = 0; i < st->datalen; i += st->info.blocksize) {
		p = (int16_t *)&st->data[i];
		tmp = addsw(p[0] * .5, p[1] * .5);
		p[0] = p[1] = (int16_t)tmp;
	}
#else /* !WITHOUTSIMD */
	asm volatile(
		"movl	%0,	%%ecx\n\t"
		"movl	%1,	%%edx\n"
	    "loop:\n\t"
		"movdqa	(%%ecx),	%%xmm0\n\t"
		"movdqa	%%xmm0,	%%xmm1\n\t"
		"pshuflw	$0xb1,	%%xmm0,	%%xmm1\n\t"
		"pshufhw	$0xb1,	%%xmm1,	%%xmm1\n\t"
		"psraw	$1,	%%xmm0\n\t"
		"psraw	$1,	%%xmm1\n\t"
		"paddsw	%%xmm1,	%%xmm0\n\t"
		"movdqa	%%xmm0,	(%%ecx)\n\t"
		"addl	$16,	%%ecx\n\t"
		"subl	$16,	%%edx\n\t"
		"cmpl	$0,	%%edx\n\t"
		"jg	loop\n"
		:
		:   "g"(st->data), "g"(st->datalen)
		:   "%ecx", "%edx", "%xmm0", "%xmm1"
	);
#endif /* WITHOUTSIMD */
	rdtsc(t1);
	if(writeWavFile(st, fo) < 0) {
		fprintf(stderr, "%s: Failed to write\n", argv[2]);
		exit(1);
	}
	fclose(fi);
	fclose(fo);
	printf("Elapsed cycles: %lld\n", t1 - t0);
	return 0;
}

