#include <sys/types.h>
#include <stdio.h>
#include <stdlib.h>
#include <wav.h>

#define DELAY .5

void
usage(char *argv0)
{

	fprintf(stderr, "usage: %s [input file] [output file]\n",
	    argv0);
}

int
main(int argc, char *argv[])
{
#ifdef WITHOUTSIMD
	int i;
	int16_t *p0, *p1;
#endif /* WITHOUTSIMD */
	int delay;
	FILE *fi, *fo;
	uint64_t t0, t1;
	wavStorage *st;

	if(argc != 3) {
		usage(argv[0]);
		exit(1);
	}
	if((fi = fopen(argv[1], "r")) == NULL) {
		fprintf(stderr,
		    "%s: No such file or directory\n", argv[1]);
		exit(1);
	}
	if((fo = fopen(argv[2], "w")) == NULL) {
		fprintf(stderr,
		    "%s: No such file or directory\n", argv[2]);
		exit(1);
	}
	if(((st = readWavFile(fi)) == NULL)
	    || (st->info.formatid != 1) || (st->info.samplebit != 16)
	    || (st->info.nchannel != 2)) {
		fprintf(stderr, "%s: Invalid file format\n",
		    argv[1]);
		exit(1);
	}
	delay = (uint32_t)(st->info.datarate * DELAY + 15)
	    & ~(uint32_t)15;
	rdtsc(t0);
#ifdef WITHOUTSIMD
	for(i = 0; i < delay; i += st->info.blocksize) {
		p0 = (int16_t *)&st->data[i];
		p0[0] = p0[0] * .5;
		p0[1] = p0[1] * .5;
	}
	for(; i < st->datalen; i += st->info.blocksize) {
		p0 = (int16_t *)&st->data[i];
		p1 = (int16_t *)&st->data[i - delay];
		p0[0] = addsw(p0[0] * .5, p1[1] * .5);
		p0[1] = addsw(p0[1] * .5, p1[0] * .5);
	}
#else /* !WITHOUTSIMD */
	asm volatile(
		"movl	%0,	%%ecx\n\t"
		"movl	%1,	%%edx\n\t"
		"movl	%%ecx,	%%eax\n"
	    "loop0:\n\t"
		"movdqa	(%%ecx),	%%xmm0\n\t"
		"psraw	$1,	%%xmm0\n\t"
		"movdqa	%%xmm0,	(%%ecx)\n\t"
		"addl	$16,	%%ecx\n\t"
		"subl	$16,	%%edx\n\t"
		"cmpl	$0,	%%edx\n\t"
		"jg	loop0\n\t"

		"movl	%2,	%%edx\n"
	    "loop1:\n\t"
		"movdqa	(%%ecx),	%%xmm0\n\t"
		"movdqa	(%%eax),	%%xmm1\n\t"
		"psraw	$1,	%%xmm0\n\t"
		"psraw	$1,	%%xmm1\n\t"

		"movdqa	%%xmm1,	%%xmm2\n\t"
		"pshuflw	$0xb1,	%%xmm1,	%%xmm2\n\t"
		"pshufhw	$0xb1,	%%xmm2,	%%xmm2\n\t"

		"paddsw	%%xmm2,	%%xmm0\n\t"
		"movdqa	%%xmm0,	(%%ecx)\n\t"
		"addl	$16,	%%ecx\n\t"
		"addl	$16,	%%eax\n\t"
		"subl	$16,	%%edx\n\t"
		"cmpl	$0,	%%edx\n\t"
		"jg	loop1\n"
		:
		:   "g"(st->data), "g"(delay), "g"(st->datalen - delay)
		:   "%eax", "%ecx", "%edx", "%xmm0", "%xmm1", "%xmm2"
	);
#endif /* WITHOUTSIMD */
	rdtsc(t1);
	if(writeWavFile(st, fo) < 0) {
		fprintf(stderr, "%s: Failed to write\n", argv[2]);
		exit(1);
	}
	fclose(fi);
	fclose(fo);
	printf("Elapsed cycles: %lld\n", t1 - t0);
	return 0;
}

