#include <sys/types.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <wav.h>

void
dumpHex(void *data, int len)
{
	int i, j;
	uint8_t *p = data;

	for(i = 0; i < len; i += 16) {
		printf("%08x: ", i);
		for(j = i; (j < i + 16) && (j < len); j++)
			printf("%02x ", p[j]);
		printf("\n");
	}
}

inline int16_t
addsw(int16_t x, int16_t y)
{
	int32_t tmp = (int32_t)x + y;

	return (tmp < -32768) ? -32768 : (tmp > 32767) ? 32767 : tmp;
}

inline int16_t
subsw(int16_t x, int16_t y)
{
	int32_t tmp = (int32_t)x - y;

	return (tmp < -32768) ? -32768 : (tmp > 32767) ? 32767 : tmp;
}

int
readRiffHeader(FILE *f, riffHeader *hdr)
{

	if(fread(hdr, sizeof(riffHeader), 1, f) != 1) {
		fprintf(stderr, "failed to read riff header\n");
		return -1;
	}
	return 0;
}

int
readChunkHeader(FILE *f, riffChunkHeader *chunk)
{

	if(fread(chunk, sizeof(riffChunkHeader), 1, f) != 1) {
		fprintf(stderr, "failed to read chunk header\n");
		return -1;
	}
	return 0;
}

wavStorage *
readWavFile(FILE *f)
{
	void *buf;
	uint32_t len;
	riffHeader fhdr;
	riffChunkHeader chdr;
	wavFormatInfo *fi;
	wavStorage *ws;

	fi = NULL;
	ws = NULL;
	if((readRiffHeader(f, &fhdr) < 0)
	    || (strncmp(fhdr.fmt, "WAVE", 4) != 0)) {
		fprintf(stderr, "invalid riff type\n");
		return NULL;
	}
	for(len = sizeof(riffHeader); len < fhdr.len + 8;) {
		if(readChunkHeader(f, &chdr) < 0) {
			fprintf(stderr, "invalid chunk type\n");
			return NULL;
		}
		len += 8;
		if((buf = malloc(chdr.len)) == NULL) {
			fprintf(stderr, "failed to allocate heap\n");
			return NULL;
		}
		if(fread(buf, chdr.len, 1, f) != 1) {
			fprintf(stderr, "failed to read chunk body\n");
			free(buf);
			return NULL;
		}
		if(strncmp(chdr.ids, "fmt ", 4) == 0) {
			if((fi = (wavFormatInfo *)
			    malloc(sizeof(wavFormatInfo))) == NULL) {
				fprintf(stderr,
				    "failed to allocate heap\n");
				free(buf);
				return NULL;
			}
			bcopy(buf, fi, sizeof(wavFormatInfo));
		}
		if(strncmp(chdr.ids, "data", 4) == 0) {
			if(fi == NULL) {
				fprintf(stderr,
				    "missing format header\n");
				free(buf);
				return NULL;
			}
			if((ws = (wavStorage *)valloc((sizeof(wavStorage)
			    + chdr.len + 4095) & ~4095)) == NULL) {
				fprintf(stderr,
				    "failed to allocate heap\n");
				free(buf);
				return NULL;
			}
			ws->infolen = sizeof(wavFormatInfo);
			ws->datalen = chdr.len;
			bcopy(fi, &ws->info, sizeof(wavFormatInfo));
			free(fi);
			bcopy(buf, &ws->data, chdr.len);
			free(buf);
			return ws;
		}
		free(buf);
		len += chdr.len;
	}
	fprintf(stderr, "missing data chunk\n");
	return NULL;
}

void
freeWavStorage(wavStorage *p)
{

	free(p);
}

int
writeWavFile(wavStorage *p, FILE *f)
{
	riffHeader fhdr;
	riffChunkHeader chdr;

	bcopy("RIFF", &fhdr.ids, 4);
	fhdr.len = p->infolen + p->datalen + 20;
	bcopy("WAVE", &fhdr.fmt, 4);
	if(fwrite(&fhdr, sizeof(riffHeader), 1, f) != 1) {
		fprintf(stderr, "failed to write riff header\n");
		return -1;
	}
	bcopy("fmt ", &chdr.ids, 4);
	chdr.len = p->infolen;
	if(fwrite(&chdr, sizeof(riffChunkHeader), 1, f) != 1) {
		fprintf(stderr, "failed to write chunk header\n");
		return -1;
	}
	if(fwrite(&p->info, sizeof(wavFormatInfo), 1, f) != 1) {
		fprintf(stderr, "failed to write file info\n");
		return -1;
	}
	bcopy("data", &chdr.ids, 4);
	chdr.len = p->datalen;
	if(fwrite(&chdr, sizeof(riffChunkHeader), 1, f) != 1) {
		fprintf(stderr, "failed to write chunk header\n");
		return -1;
	}
	if(fwrite(p->data, p->datalen, 1, f) != 1) {
		fprintf(stderr, "failed to write PCM data\n");
		return -1;
	}
	return 0;
}

