#include <stdio.h>
#include "libcom.h"

void
usage(char *p)
{

    fprintf(stderr, "usage: %s [arg1] [arg2]\n", p);
}

int
main(int argc, char *argv[])
{
    operand		   *src0, *src1, *dst;

/*
 *	check if 2 command line arguments are given
 */
    if(argc != 3) {
	usage(argv[0]);
	return -1;
    }

/*
 *	convert each command line argument to packed value
 */
    if((src0 = hex2operand128(argv[1], 16, OPERAND_ALIGNED)) == NULL) {
	fprintf(stderr, "arg1(%s): invalid value.\n", argv[1]);
	return -1;
    }
    if((src1 = hex2operand128(argv[2], 16, OPERAND_ALIGNED)) == NULL) {
	fprintf(stderr, "arg1(%s): invalid value.\n", argv[1]);
	return -1;
    }
    if((dst = alloc_operand128(16, OPERAND_ALIGNED)) == NULL) {
	fprintf(stderr, "cannot allocate memory.\n");
	return -1;
    }

/*
 *	show the contents of operands
 */
    printf("absolute difference:\n     source: %s\ndestination: %s\n",
	    operand2str(src0), operand2str(src1));

/*
 *	do the calculation with using SIMD instruction
 */
    asm volatile(
	"movdqa	(%0),	%%xmm0\n\t"
	"movdqa	(%1),	%%xmm1\n\t"
	"movdqa	%%xmm0,	%%xmm2\n\t"
	"pcmpgtd	%%xmm1,	%%xmm0\n\t"
	"movdqa	%%xmm2,	%%xmm3\n\t"
	"pxor	%%xmm1,	%%xmm2\n\t"
	"pand	%%xmm0,	%%xmm2\n\t"
	"pxor	%%xmm2,	%%xmm3\n\t"
	"pxor	%%xmm2,	%%xmm1\n\t"
	"psubd	%%xmm3,	%%xmm1\n\t"
	"movdqa	%%xmm1,	(%2)\n"
	:
	:   "r"(src0->aligned), "r"(src1->aligned), "r"(dst->aligned)
	:   "%xmm0", "%xmm1", "%xmm2"
    );

/*
 *	show the result
 */
    printf("     result: %s\n", operand2str(dst));

    free_operand(src0);
    free_operand(src1);
    free_operand(dst);

    return 0;
}

