/*
 * Copyright (c) 2006 - 2007, Junya NAKATA <jnakata@kerrymansystems.jp>
 * All rights reserved.
 */

#include <stdio.h>
#include <stdlib.h>
#include "libcom.h"

#define	ZF(res)	(((res) & 0x40) != 0)
#define	PF(res)	(((res) & 0x04) != 0)
#define	CF(res)	(((res) & 0x01) != 0)
#define	RES(res)	((((res) & 0x40) >> 4) | (((res) & 0x04) >> 1) | ((res) & 0x01))

char *result[] = {"Greater than", "Less than", "Undefined", "Undefined",
	"Equal", "Undefined", "Undefined", "Unordered"};

void
usage(char *p)
{

    fprintf(stderr, "usage: %s [arg1] [arg2] [arg3] [arg4]"
	    "[arg5] [arg6] [arg7] [arg8]\n", p);
}

int
main(int argc, char *argv[])
{
    float		   *v;
    operand		   *src, *dst;
    unsigned int	    res;

/*
 *	check if 8 command line arguments are given
 */
    if(argc != 9) {
	usage(argv[0]);
	return -1;
    }

/*
 *	allocate variables for arguments
 */
    if((src = alloc_operand128(16, OPERAND_ALIGNED)) == NULL) {
	fprintf(stderr, "cannot allocate memory.\n");
	return -1;
    }
    if((dst = alloc_operand128(16, OPERAND_ALIGNED)) == NULL) {
	fprintf(stderr, "cannot allocate memory.\n");
	return -1;
    }

/*
 *	convert source argument to packed value
 */
    v = (float *)src->aligned;
    v[3] = strtof(argv[1], NULL);
    v[2] = strtof(argv[2], NULL);
    v[1] = strtof(argv[3], NULL);
    v[0] = strtof(argv[4], NULL);

/*
 *	convert destination argument to packed value
 */
    v = (float *)dst->aligned;
    v[3] = strtof(argv[5], NULL);
    v[2] = strtof(argv[6], NULL);
    v[1] = strtof(argv[7], NULL);
    v[0] = strtof(argv[8], NULL);

/*
 *	show the contents of operands
 */
    v = (float *)src->aligned;
    printf("COMISS\n     source: (%12.4f, %12.4f, %12.4f, %12.4f)\n",
	    v[3], v[2], v[1], v[0]);
    v = (float *)dst->aligned;
    printf("destination: (%12.4f, %12.4f, %12.4f, %12.4f)\n",
	    v[3], v[2], v[1], v[0]);

/*
 *	do the calculation with using SIMD instruction
 */
    asm volatile(
	"movaps	(%0),	%%xmm0\n\t"
	"movaps	(%1),	%%xmm1\n\t"
	"comiss	%%xmm0, %%xmm1\n\t"
	"pushf\n\t"
	"popl %%eax\n\t"
	"movl %%eax,	(%2)\n"
	:
	:   "r"(src->aligned), "r"(dst->aligned), "r"(&res)
	:   "%eax", "%xmm0", "%xmm1"
    );

/*
 *	show the result
 */
    printf("         ZF:  %d PF:  %d CF:  %d\t", ZF(res), PF(res), CF(res));
    printf("\"%s\"\n", result[RES(res)]);

/*
 *	show the contents of operands
 */
    v = (float *)src->aligned;
    printf("     source: (%12.4f, %12.4f, %12.4f, %12.4f)\n",
	    v[3], v[2], v[1], v[0]);
    v = (float *)dst->aligned;
    printf("destination: (%12.4f, %12.4f, %12.4f, %12.4f)\n",
	    v[3], v[2], v[1], v[0]);

/*
 *	do the calculation with using SIMD instruction
 */
    asm volatile(
	"movaps	(%1),	%%xmm0\n\t"
	"comiss	(%0),	%%xmm0\n\t"
	"pushf\n\t"
	"popl %%eax\n\t"
	"movl %%eax,	(%2)\n"
	:
	:   "r"(src->aligned), "r"(dst->aligned), "r"(&res)
	:   "%eax", "%xmm0"
    );

/*
 *	show the result
 */
    printf("         ZF:  %d PF:  %d CF:  %d\t", ZF(res), PF(res), CF(res));
    printf("\"%s\"\n", result[RES(res)]);

    free_operand(src);
    free_operand(dst);

    return 0;
}

