#include <stdio.h>
#include "libcom.h"

void
usage(char *p)
{

    fprintf(stderr, "usage: %s\n", p);
}

inline void
func_w(void *p, void *n)
{

/*
 *	do the calculation with using SIMD instruction
 */
    asm volatile(
	"pcmpeqb	%%xmm0,	%%xmm0\n\t"
	"psrlw	(%0),	%%xmm0\n\t"
	"movdqa	%%xmm0,	(%1)\n"
	:
	:   "r"(n), "r"(p)
	:   "%xmm0"
    );
}

inline void
func_d(void *p, void *n)
{

/*
 *	do the calculation with using SIMD instruction
 */
    asm volatile(
	"pcmpeqb	%%xmm0,	%%xmm0\n\t"
	"psrld	(%0),	%%xmm0\n\t"
	"movdqa	%%xmm0,	(%1)\n"
	:
	:   "r"(n), "r"(p)
	:   "%xmm0"
    );
}

inline void
func_q(void *p, void *n)
{

/*
 *	do the calculation with using SIMD instruction
 */
    asm volatile(
	"pcmpeqb	%%xmm0,	%%xmm0\n\t"
	"psrlq	(%0),	%%xmm0\n\t"
	"movdqa	%%xmm0,	(%1)\n"
	:
	:   "r"(n), "r"(p)
	:   "%xmm0"
    );
}

int
main(int argc, char *argv[])
{
    int			    i;
    operand		   *dst, *tmp;

/*
 *	check if 0 command line argument is given
 */
    if(argc != 1) {
	usage(argv[0]);
	return -1;
    }

/*
 *	initialize memory chunk
 */
    if((dst = alloc_operand128(16, OPERAND_ALIGNED)) == NULL) {
	fprintf(stderr, "cannot allocate memory.\n");
	return -1;
    }
    if((tmp = alloc_operand128(16, OPERAND_ALIGNED)) == NULL) {
	fprintf(stderr, "cannot allocate memory.\n");
	return -1;
    }

    printf("per word:\n");
    for(i = 0; i < 16; i += 3) {
/*
 *	call calculation function
 */
	*(uint32_t *)tmp->aligned = i;
    	func_w(dst->aligned, tmp->aligned);

/*
 *	show the result
 */
    	printf("     result: %s\n", operand2str(dst));
    }

    printf("per double word:\n");
    for(i = 0; i < 16; i += 3) {
/*
 *	call calculation function
 */
	*(uint32_t *)tmp->aligned = i;
    	func_d(dst->aligned, tmp->aligned);

/*
 *	show the result
 */
    	printf("     result: %s\n", operand2str(dst));
    }

    printf("per quad word:\n");
    for(i = 0; i < 16; i += 3) {
/*
 *	call calculation function
 */
	*(uint32_t *)tmp->aligned = i;
    	func_q(dst->aligned, tmp->aligned);

/*
 *	show the result
 */
    	printf("     result: %s\n", operand2str(dst));
    }

    free_operand(dst);

    return 0;
}

