#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include "libcom.h"

void
usage(char *p)
{

    fprintf(stderr, "usage: %s [arg1] [arg2]\n", p);
}

int
main(int argc, char *argv[])
{
    extern int		    errno;
    double		    src0[2], *vd;
    int			   *vi;
    operand		   *src1, *dst;

/*
 *	check if 2 command line argument is given
 */
    if(argc != 3) {
	usage(argv[0]);
	return -1;
    }

/*
 *	convert each command line argument to packed value
 */
    if((src1 = alloc_operand128(16, OPERAND_ALIGNED)) == NULL) {
	fprintf(stderr, "cannot allocate memory.\n");
	return -1;
    }
    vd = (double *)src1->aligned;
    vd[0] = src0[0] = strtod(argv[2], NULL);
    if(errno == EINVAL) {
	fprintf(stderr, "arg2(%s): invalid value.\n", argv[2]);
	return -1;
    }
    vd[1] = src0[1] = strtod(argv[1], NULL);
    if(errno == EINVAL) {
	fprintf(stderr, "arg1(%s): invalid value.\n", argv[1]);
	return -1;
    }
    if((dst = alloc_operand128(16, OPERAND_ALIGNED)) == NULL) {
	fprintf(stderr, "cannot allocate memory.\n");
	return -1;
    }
    vi = (int *)dst->aligned;

/*
 *	show the contents of operands
 */
    printf("CVTPD2DQ\n     source: (%12.4lf, %12.4lf)\n",
	    src0[1], src0[0]);

/*
 *	do the calculation with using SIMD instruction
 */
    asm volatile(
	"cvtpd2dq	(%0),	%%xmm0\n\t"
	"movdqa		%%xmm0,	(%1)\n"
	:
	:   "r"(src0), "r"(dst->aligned)
	:   "%xmm0"
    );

/*
 *	show the result
 */
    printf("     result: (%12d, %12d, %12d, %12d)\n",
	    vi[3], vi[2], vi[1], vi[0]);

/*
 *	show the contents of operands
 */
    printf("     source: (%12.4lf, %12.4lf)\n", vd[1], vd[0]);

/*
 *	do the calculation with using SIMD instruction
 */
    asm volatile(
	"movapd		(%0),	%%xmm0\n\t"
	"cvtpd2dq	%%xmm0,	%%xmm1\n\t"
	"movdqa		%%xmm1,	(%1)\n"
	:
	:   "r"(src1->aligned), "r"(dst->aligned)
	:   "%xmm0", "%xmm1"
    );

/*
 *	show the result
 */
    printf("     result: (%12d, %12d, %12d, %12d)\n",
	    vi[3], vi[2], vi[1], vi[0]);

    free_operand(src1);
    free_operand(dst);

    return 0;
}

