#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include "libcom.h"

void
usage(char *p)
{

    fprintf(stderr, "usage: %s [arg1] [arg2]\n", p);
}

int
main(int argc, char *argv[])
{
    extern int		    errno;
    double		    src0[2], *vd;
    int			    dst[2];
    operand		   *src1;

/*
 *	check if 2 command line argument is given
 */
    if(argc != 3) {
	usage(argv[0]);
	return -1;
    }

/*
 *	convert each command line argument to packed value
 */
    if((src1 = alloc_operand128(16, OPERAND_ALIGNED)) == NULL) {
	fprintf(stderr, "cannot allocate memory.\n");
	return -1;
    }
    vd = (double *)src1->aligned;
    vd[0] = src0[0] = strtod(argv[2], NULL);
    if(errno == EINVAL) {
	fprintf(stderr, "arg2(%s): invalid value.\n", argv[2]);
	return -1;
    }
    vd[1] = src0[1] = strtod(argv[1], NULL);
    if(errno == EINVAL) {
	fprintf(stderr, "arg1(%s): invalid value.\n", argv[1]);
	return -1;
    }

/*
 *	show the contents of operands
 */
    printf("CVTPD2PI\n     source: (%12.4lf, %12.4lf)\n",
	    src0[1], src0[0]);

/*
 *	do the calculation with using SIMD instruction
 */
    asm volatile(
	"cvtpd2pi	(%0),	%%mm0\n\t"
	"movq		%%mm0,	(%1)\n\t"
	"emms\n"
	:
	:   "r"(src0), "r"(dst)
	:   "%mm0"
    );

/*
 *	show the result
 */
    printf("     result: (%12d, %12d)\n", dst[1], dst[0]);

/*
 *	show the contents of operands
 */
    printf("     source: (%12.4lf, %12.4lf)\n", vd[1], vd[0]);

/*
 *	do the calculation with using SIMD instruction
 */
    asm volatile(
	"movapd		(%0),	%%xmm0\n\t"
	"cvtpd2pi	%%xmm0,	%%mm0\n\t"
	"movq		%%mm0,	(%1)\n\t"
	"emms\n"
	:
	:   "r"(src1->aligned), "r"(dst)
	:   "%mm0", "%xmm0"
    );

/*
 *	show the result
 */
    printf("     result: (%12d, %12d)\n", dst[1], dst[0]);

    free_operand(src1);

    return 0;
}

