#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include "libcom.h"

void
usage(char *p)
{

    fprintf(stderr, "usage: %s [arg1] [arg2]\n", p);
}

int
main(int argc, char *argv[])
{
    extern int		    errno;
    int			    src0[2], *vi;
    float		   *vf;
    operand		   *src1, *dst;

/*
 *	check if 2 command line argument is given
 */
    if(argc != 3) {
	usage(argv[0]);
	return -1;
    }

/*
 *	convert each command line argument to packed value
 */
    if((src1 = alloc_operand64(8, OPERAND_ALIGNED)) == NULL) {
	fprintf(stderr, "cannot allocate memory.\n");
	return -1;
    }
    vi = (int *)src1->aligned;
    vi[0] = src0[0] = strtol(argv[2], NULL, 10);
    if(errno == EINVAL) {
	fprintf(stderr, "arg2(%s): invalid value.\n", argv[2]);
	return -1;
    }
    vi[1] = src0[1] = strtol(argv[1], NULL, 10);
    if(errno == EINVAL) {
	fprintf(stderr, "arg1(%s): invalid value.\n", argv[1]);
	return -1;
    }
    if((dst = alloc_operand128(16, OPERAND_ALIGNED)) == NULL) {
	fprintf(stderr, "cannot allocate memory.\n");
	return -1;
    }
    vf = (float *)dst->aligned;

/*
 *	show the contents of operands
 */
    printf("CVTPI2PS\n     source: (%12d, %12d)\n", src0[1], src0[0]);

/*
 *	do the calculation with using SIMD instruction
 */
    asm volatile(
	"cvtpi2ps	(%0),	%%xmm0\n\t"
	"movaps		%%xmm0,	(%1)\n\t"
	"emms\n"
	:
	:   "r"(src0), "r"(dst->aligned)
	:   "%xmm0"
    );

/*
 *	show the result
 */
    printf("     result: (%12.4lf, %12.4lf)\n", vf[1], vf[0]);

/*
 *	show the contents of operands
 */
    printf("     source: (%12d, %12d)\n", vi[1], vi[0]);

/*
 *	do the calculation with using SIMD instruction
 */
    asm volatile(
	"movq		(%0),	%%mm0\n\t"
	"cvtpi2ps	%%mm0,	%%xmm0\n\t"
	"movaps		%%xmm0,	(%1)\n\t"
	"emms\n"
	:
	:   "r"(src1->aligned), "r"(dst->aligned)
	:   "%mm0", "%xmm0"
    );

/*
 *	show the result
 */
    printf("     result: (%12.4lf, %12.4lf)\n", vf[1], vf[0]);

    free_operand(src1);
    free_operand(dst);

    return 0;
}

