#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include "libcom.h"

void
usage(char *p)
{

    fprintf(stderr, "usage: %s [arg1] [arg2] [arg3] [arg4]\n", p);
}

int
main(int argc, char *argv[])
{
    extern int		    errno;
    float		    src0[4], *vf;
    int			   *vi;
    operand		   *src1, *dst;

/*
 *	check if 4 command line argument is given
 */
    if(argc != 5) {
	usage(argv[0]);
	return -1;
    }

/*
 *	convert each command line argument to packed value
 */
    if((src1 = alloc_operand128(16, OPERAND_ALIGNED)) == NULL) {
	fprintf(stderr, "cannot allocate memory.\n");
	return -1;
    }
    vf = (float *)src1->aligned;
    vf[0] = src0[0] = strtof(argv[4], NULL);
    if(errno == EINVAL) {
	fprintf(stderr, "arg4(%s): invalid value.\n", argv[4]);
	return -1;
    }
    vf[1] = src0[1] = strtof(argv[3], NULL);
    if(errno == EINVAL) {
	fprintf(stderr, "arg3(%s): invalid value.\n", argv[3]);
	return -1;
    }
    vf[2] = src0[2] = strtof(argv[2], NULL);
    if(errno == EINVAL) {
	fprintf(stderr, "arg2(%s): invalid value.\n", argv[2]);
	return -1;
    }
    vf[3] = src0[3] = strtof(argv[1], NULL);
    if(errno == EINVAL) {
	fprintf(stderr, "arg1(%s): invalid value.\n", argv[1]);
	return -1;
    }
    if((dst = alloc_operand128(16, OPERAND_ALIGNED)) == NULL) {
	fprintf(stderr, "cannot allocate memory.\n");
	return -1;
    }
    vi = (int *)dst->aligned;

/*
 *	show the contents of operands
 */
    printf("CVTPS2DQ\n     source: (%12.4f, %12.4f, %12.4f, %12.4f)\n",
	    src0[3], src0[2], src0[1], src0[0]);

/*
 *	do the calculation with using SIMD instruction
 */
    asm volatile(
	"cvtps2dq	(%0),	%%xmm0\n\t"
	"movdqa		%%xmm0,	(%1)\n"
	:
	:   "r"(src0), "r"(dst->aligned)
	:   "%xmm0"
    );

/*
 *	show the result
 */
    printf("     result: (%12d, %12d, %12d, %12d)\n",
	    vi[3], vi[2], vi[1], vi[0]);

/*
 *	show the contents of operands
 */
    printf("     source: (%12.4f, %12.4f, %12.4f, %12.4f)\n",
	    vf[3], vf[2], vf[1], vf[0]);

/*
 *	do the calculation with using SIMD instruction
 */
    asm volatile(
	"movaps		(%0),	%%xmm0\n\t"
	"cvtps2dq	%%xmm0,	%%xmm1\n\t"
	"movdqa		%%xmm1,	(%1)\n"
	:
	:   "r"(src1->aligned), "r"(dst->aligned)
	:   "%xmm0", "%xmm1"
    );

/*
 *	show the result
 */
    printf("     result: (%12d, %12d, %12d, %12d)\n",
	    vi[3], vi[2], vi[1], vi[0]);

    free_operand(src1);
    free_operand(dst);

    return 0;
}

