#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include "libcom.h"

void
usage(char *p)
{

    fprintf(stderr, "usage: %s [arg1]\n", p);
}

int
main(int argc, char *argv[])
{
    extern int		    errno;
    int			    dst;
    double		    src0, *vd;
    operand		   *src1;

/*
 *	check if 1 command line argument is given
 */
    if(argc != 2) {
	usage(argv[0]);
	return -1;
    }

/*
 *	convert each command line argument to packed value
 */
    src0 = strtod(argv[1], NULL);
    if(errno == EINVAL) {
	fprintf(stderr, "arg1(%s): invalid value.\n", argv[1]);
	return -1;
    }
    if((src1 = alloc_operand128(16, OPERAND_ALIGNED)) == NULL) {
	fprintf(stderr, "cannot allocate memory.\n");
	return -1;
    }
    vd = (double *)src1->aligned;
    vd[0] = src0;

/*
 *	show the contents of operands
 */
    printf("CVTSD2SI\n     source:  %12.4lf\n", src0);

/*
 *	do the calculation with using SIMD instruction
 */
    asm volatile(
	"cvtsd2si	%0,	%%ecx\n\t"
	"movl		%%ecx,	(%1)\n"
	:
	:   "m"(src0), "r"(&dst)
	:   "%xmm0"
    );

/*
 *	show the result
 */
    printf("     result:  %12d\n", dst);

/*
 *	show the contents of operands
 */
    vd = (double *)src1->aligned;
    printf("     source: (%12.4lf, %12.4lf)\n", vd[1], vd[0]);

/*
 *	do the calculation with using SIMD instruction
 */
    asm volatile(
	"movapd		(%0),	%%xmm0\n\t"
	"cvtsd2si	%%xmm0,	%%ecx\n\t"
	"movl		%%ecx,	(%1)\n"
	:
	:   "r"(src1->aligned), "r"(&dst)
	:   "%ecx", "%xmm0"
    );

/*
 *	show the result
 */
    printf("     result:  %12d\n", dst);

    free_operand(src1);

    return 0;
}

