#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include "libcom.h"

void
usage(char *p)
{

    fprintf(stderr, "usage: %s [arg1]\n", p);
}

int
main(int argc, char *argv[])
{
    extern int		    errno;
    int			    src;
    double		   *v;
    operand		   *dst;

/*
 *	check if 1 command line argument is given
 */
    if(argc != 2) {
	usage(argv[0]);
	return -1;
    }

/*
 *	convert each command line argument to packed value
 */
    src = strtol(argv[1], NULL, 10);
    if(errno == EINVAL) {
	fprintf(stderr, "arg1(%s): invalid value.\n", argv[1]);
	return -1;
    }
    if((dst = alloc_operand128(16, OPERAND_ALIGNED)) == NULL) {
	fprintf(stderr, "cannot allocate memory.\n");
	return -1;
    }
    v = (double *)dst->aligned;

/*
 *	show the contents of operands
 */
    printf("CVTSI2SD\n     source:  %12d\n", src);

/*
 *	do the calculation with using SIMD instruction
 */
    asm volatile(
	"xorpd		%%xmm0,	%%xmm0\n\t"
	"cvtsi2sd	%0,	%%xmm0\n\t"
	"movapd		%%xmm0,	(%1)\n"
	:
	:   "m"(src), "r"(dst->aligned)
	:   "%xmm0"
    );

/*
 *	show the result
 */
    printf("     result: (%12.4lf, %12.4lf)\n", v[1], v[0]);

/*
 *	show the contents of operands
 */
    printf("     source:  %12d\n", src);

/*
 *	do the calculation with using SIMD instruction
 */
    asm volatile(
	"xorpd		%%xmm0,	%%xmm0\n\t"
	"movl		%0,	%%ecx\n\t"
	"cvtsi2sd	%%ecx,	%%xmm0\n\t"
	"movapd		%%xmm0,	(%1)\n"
	:
	:   "m"(src), "r"(dst->aligned)
	:   "%ecx", "%xmm0"
    );

/*
 *	show the result
 */
    printf("     result: (%12.4lf, %12.4lf)\n", v[1], v[0]);

    free_operand(dst);

    return 0;
}

