#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include "libcom.h"

void
usage(char *p)
{

    fprintf(stderr, "usage: %s [arg1]\n", p);
}

int
main(int argc, char *argv[])
{
    extern int		    errno;
    double		   *vd;
    float		    src0, *vf;
    operand		   *src1, *dst;

/*
 *	check if 1 command line argument is given
 */
    if(argc != 2) {
	usage(argv[0]);
	return -1;
    }

/*
 *	convert each command line argument to packed value
 */
    src0 = strtof(argv[1], NULL);
    if(errno == EINVAL) {
	fprintf(stderr, "arg1(%s): invalid value.\n", argv[1]);
	return -1;
    }
    if((src1 = alloc_operand128(16, OPERAND_ALIGNED)) == NULL) {
	fprintf(stderr, "cannot allocate memory.\n");
	return -1;
    }
    vf = (float *)src1->aligned;
    vf[0] = src0;
    if((dst = alloc_operand128(16, OPERAND_ALIGNED)) == NULL) {
	fprintf(stderr, "cannot allocate memory.\n");
	return -1;
    }
    vd = (double *)dst->aligned;

/*
 *	show the contents of operands
 */
    printf("CVTSS2SD\n     source:  %12.4f\n", src0);

/*
 *	do the calculation with using SIMD instruction
 */
    asm volatile(
	"xorps		%%xmm0,	%%xmm0\n\t"
	"cvtss2sd	%0,	%%xmm0\n\t"
	"movapd		%%xmm0,	(%1)\n"
	:
	:   "m"(src0), "r"(dst->aligned)
	:   "%xmm0"
    );

/*
 *	show the result
 */
    printf("     result: (%12.4lf, %12.4lf)\n", vd[1], vd[0]);

/*
 *	show the contents of operands
 */
    printf("     source:  %12.4f\n", vf[0]);

/*
 *	do the calculation with using SIMD instruction
 */
    asm volatile(
	"movaps		(%0),	%%xmm0\n\t"
	"cvtss2sd	%%xmm0,	%%xmm1\n\t"
	"movapd		%%xmm1,	(%1)\n"
	:
	:   "r"(src1->aligned), "r"(dst->aligned)
	:   "%xmm0", "%xmm1"
    );

/*
 *	show the result
 */
    printf("     result: (%12.4lf, %12.4lf)\n", vd[1], vd[0]);

    free_operand(dst);

    return 0;
}

