#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include "libcom.h"

void
usage(char *p)
{

    fprintf(stderr, "usage: %s [arg1] [arg2]\n", p);
}

int
main(int argc, char *argv[])
{
    extern int		    errno;
    float		    src0[2], *vf;
    int			   *vi;
    operand		   *src1, *dst;

/*
 *	check if 2 command line argument is given
 */
    if(argc != 3) {
	usage(argv[0]);
	return -1;
    }

/*
 *	convert each command line argument to packed value
 */
    if((src1 = alloc_operand128(16, OPERAND_ALIGNED)) == NULL) {
	fprintf(stderr, "cannot allocate memory.\n");
	return -1;
    }
    vf = (float *)src1->aligned;
    vf[0] = src0[0] = strtof(argv[2], NULL);
    if(errno == EINVAL) {
	fprintf(stderr, "arg2(%s): invalid value.\n", argv[2]);
	return -1;
    }
    vf[1] = src0[1] = strtof(argv[1], NULL);
    if(errno == EINVAL) {
	fprintf(stderr, "arg1(%s): invalid value.\n", argv[1]);
	return -1;
    }
    if((dst = alloc_operand64(8, OPERAND_ALIGNED)) == NULL) {
	fprintf(stderr, "cannot allocate memory.\n");
	return -1;
    }
    vi = (int *)dst->aligned;

/*
 *	show the contents of operands
 */
    printf("CVTTPS2PI\n     source: (%12.4f, %12.4f)\n",
	    src0[1], src0[0]);

/*
 *	do the calculation with using SIMD instruction
 */
    asm volatile(
	"cvttps2pi	(%0),	%%mm0\n\t"
	"movq		%%mm0,	(%1)\n\t"
	"emms\n"
	:
	:   "r"(src0), "r"(dst->aligned)
	:   "%mm0"
    );

/*
 *	show the result
 */
    printf("     result: (%12d, %12d)\n", vi[1], vi[0]);

/*
 *	show the contents of operands
 */
    printf("     source: (%12.4f, %12.4f)\n", vf[1], vf[0]);

/*
 *	do the calculation with using SIMD instruction
 */
    asm volatile(
	"movaps		(%0),	%%xmm0\n\t"
	"cvttps2pi	%%xmm0,	%%mm0\n\t"
	"movq		%%mm0,	(%1)\n\t"
	"emms\n"
	:
	:   "r"(src1->aligned), "r"(dst->aligned)
	:   "%xmm0", "%mm0"
    );

/*
 *	show the result
 */
    printf("     result: (%12d, %12d)\n", vi[1], vi[0]);

    free_operand(src1);
    free_operand(dst);

    return 0;
}

