#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include "libcom.h"

void
usage(char *p)
{

    fprintf(stderr, "usage: %s [arg1]\n", p);
}

int
main(int argc, char *argv[])
{
    extern int		    errno;
    float		    src0, *vf;
    int			    dst;
    operand		   *src1;

/*
 *	check if 1 command line argument is given
 */
    if(argc != 2) {
	usage(argv[0]);
	return -1;
    }

/*
 *	convert each command line argument to packed value
 */
    src0 = strtof(argv[1], NULL);
    if(errno == EINVAL) {
	fprintf(stderr, "arg1(%s): invalid value.\n", argv[1]);
	return -1;
    }
    if((src1 = alloc_operand128(16, OPERAND_ALIGNED)) == NULL) {
	fprintf(stderr, "cannot allocate memory.\n");
	return -1;
    }
    vf = (float *)src1->aligned;
    vf[0] = src0;

/*
 *	show the contents of operands
 */
    printf("CVTTSS2SI\n     source:  %12.4f\n", src0);

/*
 *	do the calculation with using SIMD instruction
 */
    asm volatile(
	"cvtTss2si	(%0),	%%ecx\n\t"
	"movl		%%ecx,	(%1)\n"
	:
	:   "r"(&src0), "r"(&dst)
	:   "%ecx", "%edx"
    );

/*
 *	show the result
 */
    printf("     result:  %12d\n", dst);

/*
 *	show the contents of operands
 */
    printf("     source:  %12.4f\n", vf[0]);

/*
 *	do the calculation with using SIMD instruction
 */
    asm volatile(
	"movaps		(%0),	%%xmm0\n\t"
	"cvtTss2si	%%xmm0,	%%edx\n\t"
	"movl		%%edx,	(%1)\n"
	:
	:   "r"(src1->aligned), "r"(&dst)
	:   "%edx", "%xmm0"
    );

/*
 *	show the result
 */
    printf("     result:  %12d\n", dst);

    free_operand(src1);

    return 0;
}

