#include <stdio.h>
#include "libcom.h"

char		    	   *tws[] = {"Empty", "Valid"};

void
usage(char *p)
{

    fprintf(stderr, "usage: %s\n", p);
}

void
show_tagwords(uint8_t tw)
{
    int			    i;

    for(i = 0; i < 8; i++)
	printf("         R%1d: %s\n", i, tws[(tw >> i) & 1]);
}

int
main(int argc, char *argv[])
{
    operand		   *s0, *s1;

/*
 *	check if no command line argument is given
 */
    if(argc != 1) {
	usage(argv[0]);
	return -1;
    }

/*
 *	allocate heaps
 */
    if((s0 = alloc_struct(512, 512, OPERAND_ALIGNED)) == NULL) {
	fprintf(stderr, "arg1(%s): invalid value.\n", argv[1]);
	return -1;
    }
    if((s1 = alloc_struct(512, 512, OPERAND_ALIGNED)) == NULL) {
	fprintf(stderr, "arg1(%s): invalid value.\n", argv[1]);
	return -1;
    }

/*
 *	do the calculation with using SIMD instruction
 */
    asm volatile(
	"pxor	%%mm0,	%%mm0\n\t"
	"fxsave	(%0)\n\t"
	"emms\n\t"
	"fxsave	(%1)\n"
	:
	:   "r"(s0->aligned), "r"(s1->aligned)
	:   "%mm0"
    );

/*
 *	show the result
 */
    printf("EMMS\n");
    printf("before emms:\n");
    show_tagwords(s0->aligned[4]);
    printf(" after emms:\n");
    show_tagwords(s1->aligned[4]);

    free_operand(s0);
    free_operand(s1);

    return 0;
}

