#include <stdio.h>
#include <math.h>
#include "libcom.h"

char		    	   *flags[] = {"IE", "DE", "ZE", "OE", "UE",
					"PE", "DAZ", "IM", "DM", "ZM",
					"OM", "UM", "PM", "RC0", "RC1",
					"FZ"};

void
usage(char *p)
{

    fprintf(stderr, "usage: %s\n", p);
}

void
show_flags(uint32_t mxcsr)
{
    int			    i;

    printf("      ");
    for(i = 0; i < 13; i++)
	printf(((i % 8) == 7) ? "%5s: %d\n      " : "%5s: %d",
		flags[i], (mxcsr >> i) & 1);
	printf("   RC: %d   FZ: %d\n", (mxcsr >> 13) & 3, mxcsr >> 15);
}

int
main(int argc, char *argv[])
{
    double		   *p;
    operand		   *s0, *s1;
    uint32_t		    r0, r1, mask = 0x1d80;

/*
 *	check if 0 command line argument is given
 */
    if(argc != 1) {
	usage(argv[0]);
	return -1;
    }

/*
 *	allocate heaps
 */
    if((s0 = alloc_struct(16, 16, OPERAND_ALIGNED)) == NULL) {
	fprintf(stderr, "arg1(%s): invalid value.\n", argv[1]);
	return -1;
    }
    p = (double *)s0->aligned;
    p[0] = p[1] = 1.0;
    if((s1 = alloc_struct(16, 16, OPERAND_ALIGNED)) == NULL) {
	fprintf(stderr, "arg1(%s): invalid value.\n", argv[1]);
	return -1;
    }
    p = (double *)s1->aligned;
    p[0] = p[1] = 0.0;

/*
 *	do the calculation with using SIMD instruction
 */
    asm volatile(
	"movapd	(%1),	%%xmm0\n\t"
	"movapd	(%2),	%%xmm1\n\t"
	"stmxcsr	(%3)\n"
	"ldmxcsr	(%0)\n\t"
	"stmxcsr	(%4)\n"
	:
	:   "r"(&mask), "r"(s0->aligned), "r"(s1->aligned),
		"r"(&r0), "r"(&r1)
	:   "%xmm0"
    );

/*
 *	show the result
 */
    printf("LDMXCSR\n");
    printf("    initial:\n");
    show_flags(r0);
    printf("     result:\n");
    show_flags(r1);

/*
 *	do the calculation with using SIMD instruction
 */
    asm volatile(
	"divpd	%%xmm1,	%%xmm0\n"
	:
	:   
	:   "%xmm0"
    );

    free_operand(s0);
    free_operand(s1);

    return 0;
}

