#include <stdio.h>
#include "libcom.h"

void
usage(char *p)
{

    fprintf(stderr, "usage: %s [arg1]\n", p);
}

int
main(int argc, char *argv[])
{
    operand		   *src0, *src1, *src2, *src3, *dst;

/*
 *	check if 1 command line argument is given
 */
    if(argc != 2) {
	usage(argv[0]);
	return -1;
    }

/*
 *	convert each command line argument to packed value
 */
    if((src0 = hex2operand64(argv[1], 8, OPERAND_ALIGNED)) == NULL) {
	fprintf(stderr, "arg1(%s): invalid value.\n", argv[1]);
	return -1;
    }
    if((src1 = hex2operand64(argv[1], 8, OPERAND_ALIGNED)) == NULL) {
	fprintf(stderr, "arg1(%s): invalid value.\n", argv[1]);
	return -1;
    }
    if((src2 = hex2operand64(argv[1], 8, OPERAND_ALIGNED)) == NULL) {
	fprintf(stderr, "arg1(%s): invalid value.\n", argv[1]);
	return -1;
    }
    if((src3 = hex2operand64(argv[1], 8, OPERAND_ALIGNED)) == NULL) {
	fprintf(stderr, "arg1(%s): invalid value.\n", argv[1]);
	return -1;
    }
    if((dst = alloc_operand64(8, OPERAND_ALIGNED)) == NULL) {
	fprintf(stderr, "cannot allocate memory.\n");
	return -1;
    }

/*
 *	show the contents of operands
 */
    printf("LFENCE\n     source: %s\n", operand2str(src0));

/*
 *	do the calculation with using SIMD instruction
 */
    asm volatile(
	"movq	(%0),	%%mm0\n\t"
	"movq	(%1),	%%mm1\n\t"
	"movq	(%2),	%%mm2\n\t"
	"lfence\n\t"
	"movq	(%3),	%%mm3\n\t"
	"movq	%%mm3,	(%4)\n\t"
	"emms\n"
	:
	:   "r"(src0->aligned), "r"(src1->aligned),
		"r"(src2->aligned), "r"(src3->aligned),
		"r"(dst->aligned)
	:   "%mm0", "%mm1", "%mm2", "%mm3"
    );

/*
 *	show the result
 */
    printf("     result: %s\n", operand2str(dst));

    free_operand(src0);
    free_operand(src1);
    free_operand(src2);
    free_operand(src3);
    free_operand(dst);

    return 0;
}

