/*
 * Copyright (c) 2006 - 2007, Junya NAKATA <jnakata@kerrymansystems.jp>
 * All rights reserved.
 */

#include <sys/types.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "libcom.h"

extern int errno;

uint64_t	    	    zero64[1];
uint64_t	    	    zero128[2]; 
uint64_t	    	    beef64[1] = {
				0xdeadbeefdeadbeefULL
			    };
uint64_t	    	    beef128[2] = {
				0xdeadbeefdeadbeefULL,
				0xdeadbeefdeadbeefULL
			    };

operand *
alloc_struct(int length, int align, enum op_align offset)
{
    operand		   *result;

    if((result
	    = (operand *)malloc(sizeof(operand))) == NULL)
	return NULL;
    bzero(result, sizeof(operand));
    if((result->start
	    = (uint8_t *)malloc(sizeof(uint8_t) * length + align)) == NULL)
	return NULL;
    bzero(result->start, sizeof(uint8_t) * length + align);
    result->aligned
	    = (uint8_t *)(((uint32_t)(result->start + align - 1)
		    & ~(align - 1)) + (offset == OPERAND_UNALIGNED));
    if((result->str
	    = (char *)malloc(sizeof(char) * length * 2 + 1)) == NULL)
	return NULL;
    result->length = length;
    return result;
}

operand *
alloc_operand64(int align, enum op_align offset)
{
    operand		   *result;

    result = alloc_struct(8, align, offset);
    return result;
}

operand *
alloc_operand128(int align, enum op_align offset)
{
    operand		   *result;

    result = alloc_struct(16, align, offset);
    return result;
}

operand *
hex2operand64(char *str, int align, enum op_align offset)
{
    int			    slen;
    uint64_t		   *v;
    operand		   *result;

    if((result = alloc_struct(8, align, offset)) == NULL)
	return NULL;
    v = (uint64_t *)result->aligned;
    slen = strlen(str);
    *v = strtoull(str, NULL, 16);
    if(errno) {
	free_operand(result);
	result = NULL;
    }
    return result;
}

operand *
hex2operand128(char *str, int align, enum op_align offset)
{
    char		    tmp;
    int			    slen;
    uint64_t		   *v;
    operand		   *result;

    if((result = alloc_struct(16, align, offset)) == NULL)
	return NULL;
    v = (uint64_t *)result->aligned;
    slen = strlen(str);
    if(slen > 16) {
	v[0] = strtoull(str + slen - 16, NULL, 16);
	tmp = *(str + slen - 16);
	*(str + slen - 16) = '\0';
	v[1] = strtoull(str, NULL, 16);
	*(str + slen - 16) = tmp;
    } else {
	v[0] = strtoull(str, NULL, 16);
    }
    if(errno) {
	free_operand(result);
	result = NULL;
    }
    return result;
}

char *
operand2str(operand *p)
{
    int			    i;

    for(i = 0; i < p->length; i++)
	if(snprintf(p->str + i * 2, 3, "%02x",
		p->aligned[p->length - i - 1]) != 2)
	    return NULL;
    return p->str;
}

void
free_operand(operand *p)
{

    free(p->start);
    free(p->str);
    free(p);
}

