#include <stdio.h>
#include "libcom.h"

void
usage(char *p)
{

    fprintf(stderr, "usage: %s [arg1] [arg2]\n", p);
}

int
main(int argc, char *argv[])
{
    operand		   *src, *msk, *dst;

/*
 *	check if 2 command line arguments are given
 */
    if(argc != 3) {
	usage(argv[0]);
	return -1;
    }

/*
 *	convert each command line argument to packed value
 */
    if((src = hex2operand128(argv[1], 16, OPERAND_ALIGNED)) == NULL) {
	fprintf(stderr, "arg1(%s): invalid value.\n", argv[1]);
	return -1;
    }
    if((msk = hex2operand128(argv[2], 16, OPERAND_ALIGNED)) == NULL) {
	fprintf(stderr, "arg2(%s): invalid value.\n", argv[2]);
	return -1;
    }
    if((dst = alloc_operand128(16, OPERAND_ALIGNED)) == NULL) {
	fprintf(stderr, "cannot allocate memory.\n");
	return -1;
    }

/*
 *	show the contents of operands
 */
    printf("MASKMOVDQU\n     source: %s\n       mask: %s\n"
	    "destination: %s\n", operand2str(src), operand2str(msk),
	    operand2str(dst));

/*
 *	do the calculation with using SIMD instruction
 */
    asm volatile(
	"movaps	(%0),	%%xmm0\n\t"
	"movaps	(%1),	%%xmm1\n\t"
	"movl	%2,	%%edi\n\t"
	"maskmovdqu	%%xmm1,	%%xmm0\n\t"
	"emms\n"
	:
	:   "r"(src->aligned), "r"(msk->aligned), "r"(dst->aligned)
	:   "%edi", "%xmm0", "%xmm1"
    );

/*
 *	show the result
 */
    printf("     result: %s\n", operand2str(dst));

    free_operand(src);
    free_operand(dst);

    return 0;
}

