/*
 * Copyright (c) 2006 - 2007, Junya NAKATA <jnakata@kerrymansystems.jp>
 * All rights reserved.
 */

#include <stdio.h>
#include <stdlib.h>
#include "libcom.h"

void
usage(char *p)
{

    fprintf(stderr, "usage: %s [arg1] [arg2] [arg3] [arg4]\n", p);
}

int
main(int argc, char *argv[])
{
    float		   *v;
    operand		   *src, *dst;

/*
 *	check if 4 command line arguments are given
 */
    if(argc != 5) {
	usage(argv[0]);
	return -1;
    }

/*
 *	convert each command line argument to packed value
 */
    if((src = alloc_operand128(16, OPERAND_ALIGNED)) == NULL) {
	fprintf(stderr, "cannot allocate memory.\n");
	return -1;
    }
    if((dst = alloc_operand128(16, OPERAND_ALIGNED)) == NULL) {
	fprintf(stderr, "cannot allocate memory.\n");
	return -1;
    }
    v = (float *)src->aligned;
    v[3] = strtof(argv[1], NULL);
    v[2] = strtof(argv[2], NULL);
    v[1] = strtof(argv[3], NULL);
    v[0] = strtof(argv[4], NULL);

/*
 *	show the contents of operands
 */
    printf("MOVAPS\n     source: (%12.4f, %12.4f, %12.4f, %12.4f)\n",
	    v[3], v[2], v[1], v[0]);

/*
 *	do the calculation with using SIMD instruction
 */
    asm volatile(
	"movaps	(%0),	%%xmm0\n\t"
	"movaps	%%xmm0,	%%xmm1\n\t"
	"movaps	%%xmm1,	(%1)\n"
	:
	:   "r"(src->aligned), "r"(dst->aligned)
	:   "%xmm0", "%xmm1"
    );

/*
 *	show the result
 */
    v = (float *)dst->aligned;
    printf("     result: (%12.4f, %12.4f, %12.4f, %12.4f)\n",
	    v[3], v[2], v[1], v[0]);

    free_operand(src);
    free_operand(dst);

    return 0;
}

