/*
 * Copyright (c) 2006 - 2007, Junya NAKATA <jnakata@kerrymansystems.jp>
 * All rights reserved.
 */

#include <stdio.h>
#include <stdlib.h>
#include "libcom.h"

void
usage(char *p)
{

    fprintf(stderr, "usage: %s [arg1] [arg2]\n", p);
}

int
main(int argc, char *argv[])
{
    int			    dst;
    double		   *v;
    operand		   *src;

/*
 *	check if 2 command line arguments are given
 */
    if(argc != 3) {
	usage(argv[0]);
	return -1;
    }

/*
 *	convert each command line argument to packed value
 */
    if((src = alloc_operand128(16, OPERAND_ALIGNED)) == NULL) {
	fprintf(stderr, "cannot allocate memory.\n");
	return -1;
    }
    v = (double *)src->aligned;
    v[1] = strtod(argv[1], NULL);
    v[0] = strtod(argv[2], NULL);

/*
 *	show the contents of operands
 */
    printf("MOVMSKPD\n     source: (%12.4f, %12.4f)\n", v[1], v[0]);

/*
 *	do the calculation with using SIMD instruction
 */
    asm volatile(
	"movapd	(%0),	%%xmm0\n\t"
	"movmskpd   %%xmm0, %%eax\n\t"
	"movl	%%eax, (%1)\n"
	:
	:   "r"(src->aligned), "r"(&dst)
	:   "%eax", "%xmm0"
    );

/*
 *	show the result
 */
    printf("     result: %04x\n", dst);

    free_operand(src);

    return 0;
}

