/*
 * Copyright (c) 2006 - 2007, Junya NAKATA <jnakata@kerrymansystems.jp>
 * All rights reserved.
 */

#include <stdio.h>
#include <stdlib.h>
#include "libcom.h"

void
usage(char *p)
{

    fprintf(stderr, "usage: %s [arg1]\n", p);
}

int
main(int argc, char *argv[])
{
    float		    src, dst, *v;
    operand		   *tmp;

/*
 *	check if 1 command line argument is given
 */
    if(argc != 2) {
	usage(argv[0]);
	return -1;
    }

/*
 *	convert each command line argument to packed value
 */
    if((tmp = alloc_operand128(16, OPERAND_ALIGNED)) == NULL) {
	fprintf(stderr, "cannot allocate memory.\n");
	return -1;
    }
    src = strtof(argv[1], NULL);

/*
 *	show the contents of operands
 */
    printf("MOVSS\n     source: %12.4f\n", src);

/*
 *	do the calculation with using SIMD instruction
 */
    asm volatile(
	"xorps	%%xmm0,	%%xmm0\n\t"
	"movss	(%0),	%%xmm0\n\t"
	"movss	%%xmm0, %%xmm1\n\t"
	"movaps	%%xmm0, (%1)\n\t"
	"movss	%%xmm1, (%2)\n"
	:
	:   "r"(&src), "r"(tmp->aligned), "r"(&dst)
	:   "%xmm0", "%xmm1"
    );

/*
 *	show the result
 */
    v = (float *)tmp->aligned;
    printf("   temporal: (%12.4f, %12.4f, %12.4f, %12.4f)\n",
	    v[3], v[2], v[1], v[0]);

    printf("     result: %12.4f\n", dst);

    free_operand(tmp);

    return 0;
}

