/*
 * Copyright (c) 2006 - 2007, Junya NAKATA <jnakata@kerrymansystems.jp>
 * All rights reserved.
 */

#include <stdio.h>
#include <stdlib.h>
#include "libcom.h"

#define	ALIGNMENT	16
#define	NUMOFELEM	65536
#define	ITERATION	8

#define rdtsc(t) asm volatile("rdtsc" : "=A" (t))

long long
movaps(int *head, int *tail)
{
    long long		    s, e;

    rdtsc(s);
    asm volatile(
	"movl	%0,	%%ecx\n"
	"loop0:\n\t"
	"movaps	(%%ecx),	%%xmm0\n\t"
	"addl	$16,	%%ecx\n\t"
	"cmpl	%1,	%%ecx\n\t"
	"jle	loop0\n\t"
	:
	:   "r"(head), "r"(tail)
	:   "%ecx", "%xmm0"
    );
    rdtsc(e);
    return e - s;
}

long long
movups(int *head, int *tail)
{
    long long		    s, e;

    rdtsc(s);
    asm volatile(
	"movl	%0,	%%ecx\n"
	"loop1:\n\t"
	"movups	(%%ecx),	%%xmm0\n\t"
	"addl	$16,	%%ecx\n\t"
	"cmpl	%1,	%%ecx\n\t"
	"jle	loop1\n\t"
	:
	:   "r"(head), "r"(tail)
	:   "%ecx", "%xmm0"
    );
    rdtsc(e);
    return e - s;
}

int
main(void)
{
    int			    i, mem[NUMOFELEM], *head, *tail;
    long long		    t0, t1, p0, p1;

    head = (int *)((int)((char *)mem + ALIGNMENT - 1)
	& ~(ALIGNMENT - 1));
    tail = (int *)((int)((char *)mem + NUMOFELEM - ALIGNMENT + 1)
	& ~(ALIGNMENT - 1));

    t0 = t1 = 0;
    /* sweep the heap in order to get rid of cache influence */
    p0 = movaps(head, tail);
    p1 = movups(head, tail);
    /* execute the process repetitively */
    for(i = 0; i < ITERATION; i++) {
	printf("movaps took %lld cycles.\n", p0 = movaps(head, tail));
	t0 += p0;
	printf("movups took %lld cycles.\n", p1 = movups(head, tail));
	t1 += p1;
    }
    printf("movaps took %lld cycles on average.\n", t0 / ITERATION);
    printf("movups took %lld cycles on average.\n", t1 / ITERATION);

    return 0;
}

