#include <stdio.h>
#include "libcom.h"

void
usage(char *p)
{

    fprintf(stderr, "usage: %s [arg1]\n", p);
}

int
main(int argc, char *argv[])
{
    operand		   *src0, *dst0, *src1, *dst1;

/*
 *	check if 1 command line argument is given
 */
    if(argc != 2) {
	usage(argv[0]);
	return -1;
    }

/*
 *	convert each command line argument to packed value
 */
    if((src0 = hex2operand64(argv[1], 8, OPERAND_ALIGNED)) == NULL) {
	fprintf(stderr, "arg1(%s): invalid value.\n", argv[1]);
	return -1;
    }
    if((dst0 = alloc_operand64(8, OPERAND_ALIGNED)) == NULL) {
	fprintf(stderr, "cannot allocate memory.\n");
	return -1;
    }
    if((src1 = hex2operand64(argv[1], 8, OPERAND_ALIGNED)) == NULL) {
	fprintf(stderr, "arg1(%s): invalid value.\n", argv[1]);
	return -1;
    }
    if((dst1 = alloc_operand64(8, OPERAND_ALIGNED)) == NULL) {
	fprintf(stderr, "cannot allocate memory.\n");
	return -1;
    }

/*
 *	show the contents of operands
 */
    printf("PABSW\n     source: %s\n", operand2str(src0));

/*
 *	do the calculation with using SIMD instruction
 */
    asm volatile(
	"movq	(%0),	%%mm0\n\t"
	"movq	(%1),	%%mm1\n\t"
	"pabsw	%%mm0,	%%mm1\n\t"
	"movq	%%mm1,	(%1)\n\t"
	"emms\n"
	:
	:   "r"(src0->aligned), "r"(dst0->aligned)
	:   "%mm0", "%mm1"
    );

/*
 *	show the result
 */
    printf("     result: %s\n", operand2str(dst0));

    free_operand(src0);
    free_operand(dst0);

/*
 *	show the contents of operands
 */
    printf("     source: %s\n", operand2str(src1));

/*
 *	do the calculation with using SIMD instruction
 */
    asm volatile(
	"movq	(%1),	%%mm0\n\t"
	"pabsw	(%0),	%%mm0\n\t"
	"movq	%%mm0,	(%1)\n\t"
	"emms\n"
	:
	:   "r"(src1->aligned), "r"(dst1->aligned)
	:   "%mm0"
    );

/*
 *	show the result
 */
    printf("     result: %s\n", operand2str(dst1));

    free_operand(src1);
    free_operand(dst1);

    return 0;
}

