/*
 * Copyright (c) 2006 - 2007, Junya NAKATA <jnakata@kerrymansystems.jp>
 * All rights reserved.
 */

#include <stdio.h>
#include "libcom.h"

void
usage(char *p)
{

    fprintf(stderr, "usage: %s [arg1] [arg2]\n", p);
}

int
main(int argc, char *argv[])
{
    operand		   *src0, *dst0, *src1, *dst1;

/*
 *	check if 2 command line arguments are given
 */
    if(argc != 3) {
	usage(argv[0]);
	return -1;
    }

/*
 *	convert each command line argument to packed value
 */
    if((src0 = hex2operand64(argv[1], 8, OPERAND_ALIGNED)) == NULL) {
	fprintf(stderr, "arg1(%s): invalid value.\n", argv[1]);
	return -1;
    }
    if((dst0 = hex2operand64(argv[2], 8, OPERAND_ALIGNED)) == NULL) {
	fprintf(stderr, "arg2(%s): invalid value.\n", argv[2]);
	return -1;
    }
    if((src1 = hex2operand64(argv[1], 8, OPERAND_ALIGNED)) == NULL) {
	fprintf(stderr, "arg1(%s): invalid value.\n", argv[1]);
	return -1;
    }
    if((dst1 = hex2operand64(argv[2], 8, OPERAND_ALIGNED)) == NULL) {
	fprintf(stderr, "arg2(%s): invalid value.\n", argv[2]);
	return -1;
    }

/*
 *	show the contents of operands
 */
    printf("PAND\n     source: %s\ndestination: %s\n",
	    operand2str(src0), operand2str(dst0));

/*
 *	do the calculation with using SIMD instruction
 */
    asm volatile(
	"movq	(%0),	%%mm0\n\t"
	"movq	(%1),	%%mm1\n\t"
	"pand	%%mm0,	%%mm1\n\t"
	"movq	%%mm1,	(%1)\n\t"
	"emms\n"
	:
	:   "r"(src0->aligned), "r"(dst0->aligned)
	:   "%mm0", "%mm1"
    );

/*
 *	show the result
 */
    printf("     result: %s\n", operand2str(dst0));

/*
 *	show the contents of operands
 */
    printf("     source: %s\ndestination: %s\n",
	    operand2str(src1), operand2str(dst1));

/*
 *	do the calculation with using SIMD instruction
 */
    asm volatile(
	"movq	(%1),	%%mm0\n\t"
	"pand	(%0),	%%mm0\n\t"
	"movq	%%mm0,	(%1)\n\t"
	"emms\n"
	:
	:   "r"(src1->aligned), "r"(dst1->aligned)
	:   "%mm0"
    );

/*
 *	show the result
 */
    printf("     result: %s\n", operand2str(dst1));

    free_operand(src1);
    free_operand(dst1);

    return 0;
}

