/*
 * Copyright (c) 2006 - 2007, Junya NAKATA <jnakata@kerrymansystems.jp>
 * All rights reserved.
 */

#include <stdio.h>
#include "libcom.h"

void
usage(char *p)
{

    fprintf(stderr, "usage: %s [arg1]\n", p);
}

int
main(int argc, char *argv[])
{
    int			    ans[4];
    operand		   *src;

/*
 *	check if 1 command line argument is given
 */
    if(argc != 2) {
	usage(argv[0]);
	return -1;
    }

/*
 *	convert each command line argument to packed value
 */
    if((src = hex2operand64(argv[1], 8, OPERAND_ALIGNED)) == NULL) {
	fprintf(stderr, "arg1(%s): invalid value.\n", argv[1]);
	return -1;
    }

/*
 *	show the contents of operands
 */
    printf("PEXTRW\n     source: %s\n", operand2str(src));

/*
 *	do the calculation with using SIMD instruction
 */
    asm volatile(
	"movq	(%0),	%%mm0\n\t"
	"pextrw	$0,	%%mm0,	%%eax\n\t"
	"movl	%%eax,	 0(%1)\n\t"
	"pextrw	$1,	%%mm0,	%%eax\n\t"
	"movl	%%eax,	 4(%1)\n\t"
	"pextrw	$2,	%%mm0,	%%eax\n\t"
	"movl	%%eax,	 8(%1)\n\t"
	"pextrw	$3,	%%mm0,	%%eax\n\t"
	"movl	%%eax,	12(%1)\n\t"
	"emms\n"
	:
	:   "r"(src->aligned), "r"(ans)
	:   "%eax", "%mm0"
    );

/*
 *	show the result
 */
    printf("   0th word: %04x\n", ans[0]);
    printf("   1st word: %04x\n", ans[1]);
    printf("   2nd word: %04x\n", ans[2]);
    printf("   3rd word: %04x\n", ans[3]);

    free_operand(src);

    return 0;
}

