/*
 * Copyright (c) 2006 - 2007, Junya NAKATA <jnakata@kerrymansystems.jp>
 * All rights reserved.
 */

#include <stdio.h>
#include "libcom.h"

void
usage(char *p)
{

    fprintf(stderr, "usage: %s [arg1]\n", p);
}

int
main(int argc, char *argv[])
{
    int			    ans[8];
    operand		   *src;

/*
 *	check if 1 command line argument is given
 */
    if(argc != 2) {
	usage(argv[0]);
	return -1;
    }

/*
 *	convert each command line argument to packed value
 */
    if((src = hex2operand128(argv[1], 16, OPERAND_ALIGNED)) == NULL) {
	fprintf(stderr, "arg1(%s): invalid value.\n", argv[1]);
	return -1;
    }

/*
 *	show the contents of operands
 */
    printf("PEXTRW\n     source: %s\n", operand2str(src));

/*
 *	do the calculation with using SIMD instruction
 */
    asm volatile(
	"movdqa	(%0),	%%xmm0\n\t"
	"pextrw	$0,	%%xmm0,	%%eax\n\t"
	"movl	%%eax,	 0(%1)\n\t"
	"pextrw	$1,	%%xmm0,	%%eax\n\t"
	"movl	%%eax,	 4(%1)\n\t"
	"pextrw	$2,	%%xmm0,	%%eax\n\t"
	"movl	%%eax,	 8(%1)\n\t"
	"pextrw	$3,	%%xmm0,	%%eax\n\t"
	"movl	%%eax,	12(%1)\n\t"
	"pextrw	$4,	%%xmm0,	%%eax\n\t"
	"movl	%%eax,	16(%1)\n\t"
	"pextrw	$5,	%%xmm0,	%%eax\n\t"
	"movl	%%eax,	20(%1)\n\t"
	"pextrw	$6,	%%xmm0,	%%eax\n\t"
	"movl	%%eax,	24(%1)\n\t"
	"pextrw	$7,	%%xmm0,	%%eax\n\t"
	"movl	%%eax,	28(%1)\n"
	:
	:   "r"(src->aligned), "r"(ans)
	:   "%eax", "%mm0"
    );

/*
 *	show the result
 */
    printf("   0th word: %04x\n", ans[0]);
    printf("   1st word: %04x\n", ans[1]);
    printf("   2nd word: %04x\n", ans[2]);
    printf("   3rd word: %04x\n", ans[3]);
    printf("   4th word: %04x\n", ans[4]);
    printf("   5th word: %04x\n", ans[5]);
    printf("   6th word: %04x\n", ans[6]);
    printf("   7th word: %04x\n", ans[7]);

    free_operand(src);

    return 0;
}

