#include <stdio.h>
#include "libcom.h"

void
usage(char *p)
{

    fprintf(stderr, "usage: %s [arg1]\n", p);
}

int
main(int argc, char *argv[])
{
    operand		   *src, *dst0, *dst1, *dst2, *dst3;

/*
 *	check if 1 command line argument is given
 */
    if(argc != 2) {
	usage(argv[0]);
	return -1;
    }

/*
 *	convert each command line argument to packed value
 */
    if((src = hex2operand64(argv[1], 8, OPERAND_ALIGNED)) == NULL) {
	fprintf(stderr, "arg1(%s): invalid value.\n", argv[1]);
	return -1;
    }
    if((dst0 = alloc_operand64(8, OPERAND_ALIGNED)) == NULL) {
	fprintf(stderr, "cannot allocate memory.\n");
	return -1;
    }
    if((dst1 = alloc_operand64(8, OPERAND_ALIGNED)) == NULL) {
	fprintf(stderr, "cannot allocate memory.\n");
	return -1;
    }
    if((dst2 = alloc_operand64(8, OPERAND_ALIGNED)) == NULL) {
	fprintf(stderr, "cannot allocate memory.\n");
	return -1;
    }
    if((dst3 = alloc_operand64(8, OPERAND_ALIGNED)) == NULL) {
	fprintf(stderr, "cannot allocate memory.\n");
	return -1;
    }

/*
 *	show the contents of operands
 */
    printf("SFENCE\n     source: %s\n", operand2str(src));

/*
 *	do the calculation with using SIMD instruction
 */
    asm volatile(
	"movq	(%0),	%%mm0\n\t"
	"movq	%%mm0,	(%1)\n\t"
	"movq	%%mm0,	(%2)\n\t"
	"movq	%%mm0,	(%3)\n\t"
	"sfence\n\t"
	"movq	%%mm0,	(%4)\n\t"
	"emms\n"
	:
	:   "r"(src->aligned), "r"(dst0->aligned),
		"r"(dst1->aligned), "r"(dst2->aligned),
		"r"(dst3->aligned)
	:   "%mm0"
    );

/*
 *	show the result
 */
    printf("     result: %s\n", operand2str(src));

    free_operand(src);
    free_operand(dst0);
    free_operand(dst1);
    free_operand(dst2);
    free_operand(dst3);

    return 0;
}

