#include <stdio.h>
#include "libcom.h"

void
usage(char *p)
{

    fprintf(stderr, "usage: %s [arg1]\n", p);
}

int
main(int argc, char *argv[])
{
    operand		   *src, *dst;

/*
 *	check if 1 command line argument is given
 */
    if(argc != 2) {
	usage(argv[0]);
	return -1;
    }

/*
 *	convert each command line argument to packed value
 */
    if((src = hex2operand64(argv[1], 8, OPERAND_ALIGNED)) == NULL) {
	fprintf(stderr, "arg1(%s): invalid value.\n", argv[1]);
	return -1;
    }
    if((dst = alloc_struct(32, 32, OPERAND_ALIGNED)) == NULL) {
	fprintf(stderr, "cannot allocate memory.\n");
	return -1;
    }

/*
 *	show the contents of operands
 */
    printf("unpack:\n     source: %s\n", operand2str(src));

/*
 *	do the calculation with using SIMD instruction
 */
    asm volatile(
	"movq	(%0),	%%xmm0\n\t"
	"pxor	%%xmm2,	%%xmm2\n\t"
	"punpcklbw	%%xmm2,	%%xmm0\n\t"
	"movdqa	%%xmm0,	%%xmm1\n\t"
	"punpcklwd	%%xmm2,	%%xmm0\n\t"
	"punpckhwd	%%xmm2,	%%xmm1\n\t"
	"movdqa	%%xmm0,	(%1)\n"
	"movdqa	%%xmm1,	(%2)\n"
	:
	:   "r"(src->aligned), "r"(((uint64_t *)dst->aligned) + 0),
	    "r"(((uint64_t *)dst->aligned) + 2)
	:   "%xmm0", "%xmm2"
    );

/*
 *	show the result
 */
    printf("     result: %s\n", operand2str(dst));

    free_operand(src);
    free_operand(dst);

    return 0;
}

