#include <stdio.h>
#include <stdlib.h>
#include "libcom.h"

void
usage(char *p)
{

    fprintf(stderr, "usage: %s [arg01] [arg02] [arg03] [arg04]\n"
	    "\t[arg05] [arg06] [arg07] [arg08]\n"
	    "\t[arg09] [arg10] [arg11] [arg12]\n"
	    "\t[arg13] [arg14] [arg15] [arg16]\n", p);
}

int
main(int argc, char *argv[])
{
    int			    i, j;
    float		   *v;
    operand		   *src, *dst;

/*
 *	check if 16 command line arguments are given
 */
    if(argc != 17) {
	usage(argv[0]);
	return -1;
    }

/*
 *	convert each command line argument to packed value
 */
    if((src = alloc_struct(64, 64, OPERAND_ALIGNED)) == NULL) {
	fprintf(stderr, "cannot allocate memory.\n");
	return -1;
    }
    if((dst = alloc_operand128(16, OPERAND_ALIGNED)) == NULL) {
	fprintf(stderr, "cannot allocate memory.\n");
	return -1;
    }
    v = (float *)src->aligned;
    for(i = 0; i < 16; i += 4)
	for(j = 0; j < 4; j++)
    	v[i + 3 - j] = strtof(argv[i + j + 1], NULL);

/*
 *	show the contents of operands
 */
    v = (float *)src->aligned;
    printf("vertical add:\n     source:\n");
    for(i = 0; i < 16; i += 4)
    	printf("              %12.4f  %12.4f  %12.4f  %12.4f\n",
	    	v[i + 3], v[i + 2], v[i + 1], v[i + 0]);

/*
 *	do the calculation with using SIMD instruction
 */
    asm volatile(
	"movaps	 0(%0),	%%xmm0\n\t"
	"addps	16(%0),	%%xmm0\n\t"
	"addps	32(%0),	%%xmm0\n\t"
	"addps	48(%0),	%%xmm0\n\t"
	"movaps	%%xmm0,	(%1)\n"
	:
	:   "r"(src->aligned), "r"(dst->aligned)
	:   "%xmm0"
    );

/*
 *	show the result
 */
    v = (float *)dst->aligned;
    printf("     result:  %12.4f  %12.4f  %12.4f  %12.4f \n",
	    v[3], v[2], v[1], v[0]);

    free_operand(src);
    free_operand(dst);

    return 0;
}

